==> dson-codec-1.1.1-sources.jar: cn/wjybxx/dsoncodec/codecs/LocalTimeCodec.java <==
/*
 * Copyright 2023-2024 wjybxx(845740757@qq.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cn.wjybxx.dsoncodec.codecs;

import cn.wjybxx.base.time.TimeUtils;
import cn.wjybxx.dson.text.ObjectStyle;
import cn.wjybxx.dson.types.ExtDateTime;
import cn.wjybxx.dsoncodec.DsonCodec;
import cn.wjybxx.dsoncodec.DsonObjectReader;
import cn.wjybxx.dsoncodec.DsonObjectWriter;
import cn.wjybxx.dsoncodec.TypeInfo;
import cn.wjybxx.dsoncodec.annotations.DsonCodecScanIgnore;

import javax.annotation.Nonnull;
import java.time.LocalTime;
import java.util.function.Supplier;

/**
 * @author wjybxx
 * date - 2024/1/7
 */
@DsonCodecScanIgnore
public class LocalTimeCodec implements DsonCodec<LocalTime> {
    @Override
    public boolean isWriteAsArray() {
        return false;
    }

    @Override
    public boolean autoStartEnd() {
        return false;
    }

    @Nonnull
    @Override
    public Class<LocalTime> getEncoderClass() {
        return LocalTime.class;
    }

    @Override
    public void writeObject(DsonObjectWriter writer, LocalTime instance, TypeInfo<?> typeInfo, ObjectStyle style) {
        writer.writeExtDateTime(null, toDateTime(instance));
    }

    @Override
    public LocalTime readObject(DsonObjectReader reader, TypeInfo<?> typeInfo, Supplier<? extends LocalTime> factory) {
        ExtDateTime dateTime = reader.readExtDateTime(reader.getCurrentName());
        return ofDateTime(dateTime);
    }

    private static LocalTime ofDateTime(ExtDateTime dateTime) {
        long nanoOfDay = dateTime.getSeconds() * TimeUtils.NANOS_PER_SECOND + dateTime.getNanos();
        return LocalTime.ofNanoOfDay(nanoOfDay);
    }

    private static ExtDateTime toDateTime(LocalTime localTime) {
        return new ExtDateTime(localTime.toSecondOfDay(), localTime.getNano(), 0, ExtDateTime.MASK_TIME);
    }
}

==> dson-codec-1.4.0-sources.jar: cn/wjybxx/dsoncodec/codecs/LocalTimeCodec.java <==
/*
 * Copyright 2023-2024 wjybxx(845740757@qq.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cn.wjybxx.dsoncodec.codecs;

import cn.wjybxx.base.time.TimeUtils;
import cn.wjybxx.dson.text.ObjectStyle;
import cn.wjybxx.dson.types.ExtDateTime;
import cn.wjybxx.dsoncodec.DsonCodec;
import cn.wjybxx.dsoncodec.DsonObjectReader;
import cn.wjybxx.dsoncodec.DsonObjectWriter;
import cn.wjybxx.dsoncodec.TypeInfo;
import cn.wjybxx.dsoncodec.annotations.DsonCodecScanIgnore;

import javax.annotation.Nonnull;
import java.time.LocalTime;
import java.util.function.Supplier;

/**
 * @author wjybxx
 * date - 2024/1/7
 */
@DsonCodecScanIgnore
public class LocalTimeCodec implements DsonCodec<LocalTime> {

    @Nonnull
    @Override
    public TypeInfo getEncoderType() {
        return TypeInfo.of(LocalTime.class);
    }

    @Override
    public boolean autoStartEnd() {
        return false;
    }

    @Override
    public void writeObject(DsonObjectWriter writer, LocalTime inst, TypeInfo declaredType, ObjectStyle style) {
        writer.writeExtDateTime(null, toDateTime(inst));
    }

    @Override
    public LocalTime readObject(DsonObjectReader reader, Supplier<? extends LocalTime> factory) {
        ExtDateTime dateTime = reader.readExtDateTime(null);
        return ofDateTime(dateTime);
    }

    private static LocalTime ofDateTime(ExtDateTime dateTime) {
        long nanoOfDay = dateTime.getSeconds() * TimeUtils.NANOS_PER_SECOND + dateTime.getNanos();
        return LocalTime.ofNanoOfDay(nanoOfDay);
    }

    private static ExtDateTime toDateTime(LocalTime localTime) {
        return new ExtDateTime(localTime.toSecondOfDay(), localTime.getNano(), 0, ExtDateTime.MASK_TIME);
    }
}
